@@ -1,14 +1,18 @@
 package Bio::Das::Segment;
 
-# $Id: Segment.pm,v 1.20 2009/08/26 21:57:11 lstein Exp $
+# $Id: Segment.pm,v 1.23 2009/09/02 19:59:39 lstein Exp $
 use strict;
 use Bio::Root::Root;
 use Bio::Das::SegmentI;
 use Bio::Das::Util 'rearrange';
+use File::Basename 'basename';
+use Data::Dumper 'Dumper';
+use File::Spec;
+use File::Path 'mkpath';
 use vars qw(@ISA $VERSION);
 @ISA = qw(Bio::Root::Root Bio::Das::SegmentI);
 
-$VERSION = 0.90;
+$VERSION = 0.91;
 
 use overload '""' => 'asString';
 *abs_ref   = *refseq = \&ref;
@@ -222,9 +226,15 @@ sub render {
   my $self = shift;
   my ($panel,$position_to_insert,$options,$max_bump,$max_label) = @_;
 
+  $max_bump  = 50 unless defined $max_bump;
+  $max_label = 50 unless defined $max_label;
+  $options   = 0  unless defined $options;
+  $panel->key_style('between') if $panel->key_style eq 'bottom'; # bottom key doesn't work with stylesheets
+
   my @COLORS = qw(cyan blue red yellow green wheat turquoise orange);
 
-  my $stylesheet = $self->das->stylesheet;
+  # cache stylesheet
+  my $stylesheet = $self->get_cached_stylesheet;
 
   my @override = $options && CORE::ref($options) eq 'HASH' ? %$options : ();
   my @new_tracks;
@@ -272,14 +282,15 @@ sub render {
   # reconfigure bumping, etc
   for my $type (keys %type_count) {
     my $type_count = $type_count{$type};
+    warn "$type => $type_count";
     my $do_bump    = defined $track_configs{$type}{-bump} ? $track_configs{$type}{-bump}
-                     : $options == 0 ? $type_count <= $max_bump
-                     : $options == 1 ? 0
-		     : $options == 2 ? 1
-		     : $options == 3 ? 1
-		     : $options == 4 ? 2
-		     : $options == 5 ? 2
-		     : 0;
+                                                          : $options == 0 ? $type_count <= $max_bump
+							  : $options == 1 ? 0
+							  : $options == 2 ? 1
+							  : $options == 3 ? 1
+							  : $options == 4 ? 2
+							  : $options == 5 ? 2
+							  : 0;
 
     my $maxed_out  = $type_count > $max_label;
     my $conf_label = defined $track_configs{$type}{-label} 
@@ -298,11 +309,61 @@ sub render {
     $factory->set_option(connector  => 'none') if !$do_bump;
     $factory->set_option(bump       => $do_bump);
     $factory->set_option(label      => $do_label);
+
+    warn "bumping = $do_bump";
+    warn "label   = $do_label";
   }
   my $track_count = keys %tracks;
   return wantarray ? ($track_count,$panel,\@new_tracks) : $track_count;
 }
 
+sub get_cached_stylesheet {
+    my $self    = shift;
+    my $tmpdir  = File::Spec->tmpdir;
+    my $program = basename($0);
+    my $user    = (getpwuid($>))[0];
+    my $url     = $self->das->name.'/stylesheet';
+    foreach ($program,$user,$url) {
+	tr/a-zA-Z0-9_-/_/c;
+    }
+
+    my $dir  = File::Spec->catfile($tmpdir,"$program-$user");
+    mkpath($dir) or die "$dir: $!" unless -d $dir;
+    my $path = File::Spec->catfile($dir,$url);
+
+    my $stylesheet;
+
+    eval {
+
+	# cache for 5 minutes
+	my $mtime = (stat($path))[9];
+	if ($mtime && ((time() - $mtime)/60) < 5.0) {
+	    open my $f,'<',$path or die "$path: $!";
+	    my $s;
+	    $s .= $_ while <$f>;
+	    close $f;
+	    my $VAR1;
+	    $stylesheet = eval "$s; \$VAR1";
+	    warn $@ if $@;
+	    utime undef,undef,$path;
+	}
+	
+	else {
+	    $stylesheet = $self->das->stylesheet;
+	    my $d = Data::Dumper->new([$stylesheet]);
+	    $d->Purity(1);
+	    open my $f,">",$path or die "$path: $!";
+	    print $f $d->Dump;
+	    close $f;
+	}
+    
+	return $stylesheet;
+    };
+
+    # something went wrong, so revert to non-cached behavior
+    return $self->das->stylesheet;
+}
+
 1;
 
 __END__
